@@ -1,6 +1,6 @@
-/* Copyright (C) 1989, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved. */
+/* Copyright (C) 1989, 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved. */
 
-/*$Id: ansi2knr.c,v 1.14 1999/04/13 14:44:33 meyering Exp $*/
+/*$Id: ansi2knr.c,v 1.2 2000/03/10 03:16:17 lpd Exp $*/
 /* Convert ANSI C function definitions to K&R ("traditional C") syntax */
 
 /*
@@ -37,21 +37,21 @@ program under the GPL.
  * There are no error messages.
  *
  * ansi2knr recognizes function definitions by seeing a non-keyword
- * identifier at the left margin, followed by a left parenthesis,
- * with a right parenthesis as the last character on the line,
- * and with a left brace as the first token on the following line
- * (ignoring possible intervening comments), except that a line
+ * identifier at the left margin, followed by a left parenthesis, with a
+ * right parenthesis as the last character on the line, and with a left
+ * brace as the first token on the following line (ignoring possible
+ * intervening comments and/or preprocessor directives), except that a line
  * consisting of only
  *	identifier1(identifier2)
  * will not be considered a function definition unless identifier2 is
  * the word "void", and a line consisting of
  *	identifier1(identifier2, <<arbitrary>>)
  * will not be considered a function definition.
- * ansi2knr will recognize a multi-line header provided
- * that no intervening line ends with a left or right brace or a semicolon.
- * These algorithms ignore whitespace and comments, except that
- * the function name must be the first thing on the line.
- * The following constructs will confuse it:
+ * ansi2knr will recognize a multi-line header provided that no intervening
+ * line ends with a left or right brace or a semicolon.  These algorithms
+ * ignore whitespace, comments, and preprocessor directives, except that
+ * the function name must be the first thing on the line.  The following
+ * constructs will confuse it:
  *	- Any other construct that starts at the left margin and
  *	    follows the above syntax (such as a macro or function call).
  *	- Some macros that tinker with the syntax of function headers.
@@ -61,6 +61,21 @@ program under the GPL.
  * The original and principal author of ansi2knr is L. Peter Deutsch
  * <ghost@aladdin.com>.  Other authors are noted in the change history
  * that follows (in reverse chronological order):
+
+	2000-03-05  Paul Eggert  <eggert@twinsun.com>
+
+	Add support for concatenated string literals.
+	* ansi2knr.c (concatlits): New decl.
+	(main): Invoke concatlits to concatenate string literals.
+	(scanstring): Handle backslash-newline correctly.  Work with
+	character constants.  Fix bug when scanning backwards through
+	backslash-quote.  Check for unterminated strings.
+	(convert1): Parse character constants, too.
+	(appendline, concatlits): New functions.
+	* ansi2knr.1: Document this.
+
+	lpd 1999-08-17 added code to allow preprocessor directives
+		wherever comments are allowed
 	lpd 1999-04-12 added minor fixes from Pavel Roskin
 		<pavel_roskin@geocities.com> for clean compilation with
 		gcc -W -Wall
@@ -196,11 +211,15 @@ program under the GPL.
 #define isidfirstchar(ch) (is_alpha(ch) || (ch) == '_')
 
 /* Forward references */
+char *ppdirforward();
+char *ppdirbackward();
 char *skipspace();
 char *scanstring();
 int writeblanks();
 int test1();
 int convert1();
+int concatlits();
+int getinputline();
 
 /* The main program */
 int
@@ -212,7 +231,7 @@ main(argc, argv)
 	char *filename = 0;
 	char *program_name = argv[0];
 	char *output_name = 0;
-#define bufsize 5000			/* arbitrary size */
+#define bufsize 50000			/* arbitrary size */
 	char *buf;
 	char *line;
 	char *more;
@@ -283,7 +302,7 @@ main(argc, argv)
 		exit(1);
 	   }
 	line = buf;
-	while ( fgets(line, (unsigned)(buf + bufsize - line), in) != NULL )
+	while ( getinputline(line, buf + bufsize, in) )
 	   {
 test:		line += strlen(line);
 		switch ( test1(buf) )
@@ -296,9 +315,9 @@ test:		line += strlen(line);
 			more = ++line;
 f:			if ( line >= buf + (bufsize - 1) ) /* overflow check */
 			  goto wl;
-			if ( fgets(line, (unsigned)(buf + bufsize - line), in) == NULL )
+			if ( ! getinputline(line, buf + bufsize, in) )
 			  goto wl;
-			switch ( *skipspace(more, 1) )
+			switch ( *skipspace(ppdirforward(more), 1) )
 			  {
 			  case '{':
 			    /* Definitely a function header. */
@@ -324,6 +343,7 @@ f:			if ( line >= buf + (bufsize - 1) ) /* overflow check */
 			  continue;
 			/* falls through */
 		default:		/* not a function */
+			concatlits(buf, line, buf + bufsize, in);
 wl:			fputs(buf, out);
 			break;
 		   }
@@ -349,36 +369,86 @@ wl:			fputs(buf, out);
 	return 0;
 }
 
+/*
+ * Skip forward or backward over one or more preprocessor directives.
+ */
+char *
+ppdirforward(p)
+    char *p;
+{
+    for (; *p == '#'; ++p) {
+	for (; *p != '\r' && *p != '\n'; ++p)
+	    if (*p == 0)
+		return p;
+	if (*p == '\r' && p[1] == '\n')
+	    ++p;
+    }
+    return p;
+}
+char *
+ppdirbackward(p, limit)
+    char *p;
+    char *limit;
+{
+    char *np = p;
+
+    for (;; p = --np) {
+	if (*np == '\n' && np[-1] == '\r')
+	    --np;
+	for (; np > limit && np[-1] != '\r' && np[-1] != '\n'; --np)
+	    if (np[-1] == 0)
+		return np;
+	if (*np != '#')
+	    return p;
+    }
+}
+
 /* Skip over whitespace and comments, in either direction. */
 char *
 skipspace(p, dir)
-    register char *p;
-    register int dir;			/* 1 for forward, -1 for backward */
-{	for ( ; ; )
-	   {	while ( is_space(*p) )
-		  p += dir;
-		if ( !(*p == '/' && p[dir] == '*') )
-		  break;
-		p += dir;  p += dir;
-		while ( !(*p == '*' && p[dir] == '/') )
-		   {	if ( *p == 0 )
-			  return p;	/* multi-line comment?? */
-			p += dir;
-		   }
-		p += dir;  p += dir;
-	   }
-	return p;
+    char *p;
+    int dir;			/* 1 for forward, -1 for backward */
+{
+    for ( ; ; ) {
+	while ( is_space(*p) )
+	    p += dir;
+	if ( !(*p == '/' && p[dir] == '*') )
+	    break;
+	p += dir;  p += dir;
+	while ( !(*p == '*' && p[dir] == '/') ) {
+	    if ( *p == 0 )
+		return p;	/* multi-line comment?? */
+	    p += dir;
+	}
+	p += dir;  p += dir;
+    }
+    return p;
 }
 
 /* Scan over a quoted string, in either direction. */
 char *
 scanstring(p, dir)
-    register char *p;
-    register int dir;
+    char *p;
+    int dir;
 {
-    for (p += dir; ; p += dir)
-	if (*p == '"' && p[-dir] != '\\')
-	    return p + dir;
+    char quote = *p;
+
+    for (p += dir; *p; p += dir) {
+	if (*p == quote) {
+	    char *q = p;
+	    int backslashed;
+
+	    for (backslashed = 0; ; backslashed ^= 1) {
+		for (q--; *q == '\n' && q[-1] == '\\'; q -= 2)
+		    continue;
+		if (*q != '\\')
+		    break;
+	    }
+	    if (!backslashed)
+		return p + dir;
+	}
+    }
+    return p; /* unterminated string */
 }
 
 /*
@@ -389,11 +459,13 @@ int
 writeblanks(start, end)
     char *start;
     char *end;
-{	char *p;
-	for ( p = start; p < end; p++ )
-	  if ( *p != '\r' && *p != '\n' )
+{
+    char *p;
+
+    for ( p = start; p < end; p++ )
+	if ( *p != '\r' && *p != '\n' )
 	    *p = ' ';
-	return 0;
+    return 0;
 }
 
 /*
@@ -412,14 +484,14 @@ writeblanks(start, end)
 int
 test1(buf)
     char *buf;
-{	register char *p = buf;
+{	char *p = buf;
 	char *bend;
 	char *endfn;
 	int contin;
 
 	if ( !isidfirstchar(*p) )
 	  return 0;		/* no name at left margin */
-	bend = skipspace(buf + strlen(buf) - 1, -1);
+	bend = skipspace(ppdirbackward(buf + strlen(buf) - 1, buf), -1);
 	switch ( *bend )
 	   {
 	   case ';': contin = 0 /*2*/; break;
@@ -498,7 +570,7 @@ convert1(buf, out, header, convert_varargs)
     int header;			/* Boolean */
     int convert_varargs;	/* Boolean */
 {	char *endfn;
-	register char *p;
+	char *p;
 	/*
 	 * The breaks table contains pointers to the beginning and end
 	 * of each argument.
@@ -558,7 +630,7 @@ top:	p = endfn;
 				if (p[1] == '*')
 				    p = skipspace(p, 1) - 1;
 				break;
-			   case '"':
+			   case '"': case '\'':
 			       p = scanstring(p, 1) - 1;
 			       break;
 			   default:
@@ -592,7 +664,7 @@ top:	p = endfn;
 				       if (p > buf && p[-1] == '*')
 					   p = skipspace(p, -1) + 1;
 				       break;
-				   case '"':
+				   case '"': case '\'':
 				       p = scanstring(p, -1) + 1;
 				       break;
 				   default: ;
@@ -676,3 +748,137 @@ found:		if ( *p == '.' && p[-1] == '.' && p[-2] == '.' )
 	free((char *)breaks);
 	return 0;
 }
+
+/*
+ * Read an input line of C source code.  If the line is incomplete
+ * because of an unterminated comment, string, or a backslash at the
+ * end, keep reading until a completed line is found.  Terminate the
+ * line with a null byte after the newline.  Return nonzero if
+ * something was read successfully.
+ */
+int
+getinputline(buf, bufend, in)
+    char *buf;
+    char *bufend;
+    FILE *in;
+{
+    char *p = buf;
+    int ch; /* current input char */ 
+    int ch_1 = EOF; /* previous input char;
+		       EOF if at start of input;
+		       EOF-1 if backslash-backslash-newline within literal */
+    int pbs; /* count of significant backslashes immediately
+		before ch_1; valid only within literal */
+    int state = 0;
+    while (p < bufend - 1 && (ch = getc(in)) != EOF) {
+      *p++ = ch;
+      if (ch == '\r')
+	continue;
+
+      switch (state) {
+      case 0: /* initial state */
+	switch (ch) {
+	case '"': case '\'': state = ch; pbs = 0; break;
+	case '*': case '/': if (ch_1 == '/') state = ch; break;
+	}
+	break;
+
+      case '"': /* string literal */
+	if (ch == '"' && ch_1 != '\\' && ch_1 != EOF-1 && (pbs & 1) == 0)
+	  state = 0;
+	break;
+
+      case '\'': /* character literal */
+	if (ch == '\'' && ch_1 != '\\' && ch_1 != EOF-1 && (pbs & 1) == 0)
+	  state = 0;
+	break;
+
+      case '*': /* C comment */
+	if (ch == '/' && ch_1 == '*')
+	  state = 0;
+	break;
+
+      case '/': /* C++ comment */
+	break;
+      }
+
+      if (ch == '\n') {
+	if (ch_1 == '\\')
+	  /* Backslash-newline does not contribute to state.  */
+	  ch = ch_2;
+	else if (state == 0 || state == '/')
+	  break;
+      }
+      ch_2 = ch_1;
+      ch_1 = ch;
+    }
+    if (p < bufend)
+      *p = '\0';
+    return p != buf;
+}
+
+/* Append a line to a buffer.  Return the end of the appended line.  */
+char *
+appendline(lineend, bufend, in)
+    char *lineend;
+    char *bufend;
+    FILE *in;
+{
+    if (bufend == lineend)
+	return NULL;
+    if (!getinputline(lineend, bufend, in))
+	return NULL;
+    return lineend + strlen(lineend);
+}
+
+/*
+ * Concatenate string literals in a non-function line, appending
+ * new lines if a string literal crosses a line boundary or ends a line.
+ */ 
+int
+concatlits(line, lineend, bufend, in)
+    char *line;
+    char *lineend;
+    char *bufend;
+    FILE *in;
+{
+    char *d = line;
+    char *s = line;
+    char *s1;
+    int pending_newlines = 0;
+
+    if (lineend[-1] != '\n')
+	return 0;
+    if (*skipspace(s, 1) == '#')
+	return 0;
+    while (*s) {
+	switch ((*d++ = *s++))
+	  {
+	  case '"': case '\'':
+	      for (;;) {
+		  while ((s1 = scanstring(s - 1, 1)) == lineend)
+		      if (!(lineend = appendline(lineend, bufend, in)))
+			  goto finish;
+		  do {
+		      *d++ = *s++;
+		  } while (s != s1);
+		  if (s[-1] != '"')
+		      break;
+		  while ((s1 = skipspace(s, 1)) == lineend)
+		      if (!(lineend = appendline(lineend, bufend, in)))
+			  goto finish;
+		  if (*s1 != '"')
+		      break;
+		  d--;
+		  do {
+		      pending_newlines += *s++ == '\n';
+		  } while (s <= s1);
+	      }
+	  }
+    }
+    while (pending_newlines--)
+	*d++ = '\n';
+finish:
+    strcpy(d, s);
+    return 0;
+}
