@@ -1,5 +1,5 @@
 /* Common declarations for the tar program.
-   Copyright 1988, 92, 93, 94, 96, 97, 99, 2000 Free Software Foundation, Inc.
+   Copyright 1988,92,93,94,96,97,99,2000, 2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -36,7 +36,7 @@
 # define TTY_NAME "/dev/tty"
 #endif
 
-/* GLOBAL is defined to empty in `tar.c' only, and left alone in other `*.c'
+/* GLOBAL is defined to empty in tar.c only, and left alone in other *.c
    modules.  Here, we merely set it to "extern" if it is not already set.
    GNU tar does depend on the system loader to preset all GLOBAL variables to
    neutral (or zero) values, explicit initialization is usually not done.  */
@@ -58,8 +58,8 @@ GLOBAL int exit_status;
    writes a message on stderr and aborts immediately, with another message
    line telling so.  USAGE_ERROR works like FATAL_ERROR except that the
    other message line suggests trying --help.  All four macros accept a
-   single argument of the form ((0, errno, _("FORMAT"), Args...)).  `errno'
-   is `0' when the error is not being detected by the system.  */
+   single argument of the form ((0, errno, _("FORMAT"), Args...)).  errno
+   is zero when the error is not being detected by the system.  */
 
 #define WARN(Args) \
   error Args
@@ -73,11 +73,11 @@ GLOBAL int exit_status;
 /* Information gleaned from the command line.  */
 
 #include "arith.h"
-#include "backupfile.h"
-#include "basename.h"
-#include "exclude.h"
-#include "modechange.h"
-#include "safe-read.h"
+#include <backupfile.h>
+#include <exclude.h>
+#include <modechange.h>
+#include <safe-read.h>
+#include <full-write.h>
 
 /* Log base 2 of common values.  */
 #define LG_8 3
@@ -204,8 +204,8 @@ GLOBAL int multi_volume_option;
    do not get archived (also see after_date_option above).  */
 GLOBAL time_t newer_mtime_option;
 
-/* Boolean value.  */
-GLOBAL int no_recurse_option;
+/* Zero if there is no recursion, otherwise FNM_LEADING_DIR.  */
+GLOBAL int recursion_option;
 
 /* Boolean value.  */
 GLOBAL int numeric_owner_option;
@@ -315,8 +315,6 @@ struct name
     char fake;			/* dummy entry */
     char name[1];
   };
-GLOBAL struct name *namelist;	/* points to first name in list */
-GLOBAL struct name *namelast;	/* points to last name in list */
 
 /* Information about a sparse file.  */
 struct sp_array
@@ -401,10 +399,10 @@ void verify_volume PARAMS ((void));
 
 /* Module extract.c.  */
 
+extern int we_are_root;
 void extr_init PARAMS ((void));
 void extract_archive PARAMS ((void));
-void apply_delayed_set_stat PARAMS ((void));
-void apply_nonancestor_delayed_set_stat PARAMS ((char const *));
+void extract_finish PARAMS ((void));
 void fatal_exit PARAMS ((void)) __attribute__ ((noreturn));
 
 /* Module delete.c.  */
@@ -416,7 +414,7 @@ void delete_archive_members PARAMS ((void));
 char *get_directory_contents PARAMS ((char *, dev_t));
 void read_directory_file PARAMS ((void));
 void write_directory_file PARAMS ((void));
-void gnu_restore PARAMS ((int));
+void gnu_restore PARAMS ((size_t));
 
 /* Module list.c.  */
 
@@ -465,8 +463,8 @@ void print_for_mkdir PARAMS ((char *, int, mode_t));
 void print_header PARAMS ((void));
 void read_and PARAMS ((void (*do_) ()));
 enum read_header read_header PARAMS ((void));
-void skip_extended_headers PARAMS ((void));
 void skip_file PARAMS ((off_t));
+void skip_member PARAMS ((void));
 
 /* Module mangle.c.  */
 
@@ -480,7 +478,6 @@ int unquote_string PARAMS ((char *));
 
 int contains_dot_dot PARAMS ((char const *));
 
-int is_dot_or_dotdot PARAMS ((const char *));
 int remove_any_file PARAMS ((const char *, int));
 int maybe_backup_file PARAMS ((const char *, int));
 void undo_last_backup PARAMS ((void));
@@ -497,8 +494,6 @@ void chmod_error_details PARAMS ((char const *, mode_t));
 void chown_error_details PARAMS ((char const *, uid_t, gid_t));
 void close_error PARAMS ((char const *));
 void close_warn PARAMS ((char const *));
-void closedir_error PARAMS ((char const *));
-void closedir_warn PARAMS ((char const *));
 void exec_fatal PARAMS ((char const *)) __attribute__ ((noreturn));
 void mkdir_error PARAMS ((char const *));
 void mkfifo_error PARAMS ((char const *));
@@ -506,23 +501,22 @@ void mknod_error PARAMS ((char const *));
 void open_error PARAMS ((char const *));
 void open_fatal PARAMS ((char const *)) __attribute__ ((noreturn));
 void open_warn PARAMS ((char const *));
-void opendir_error PARAMS ((char const *));
-void opendir_warn PARAMS ((char const *));
 void read_error PARAMS ((char const *));
 void read_error_details PARAMS ((char const *, off_t, size_t));
 void read_fatal PARAMS ((char const *)) __attribute__ ((noreturn));
 void read_fatal_details PARAMS ((char const *, off_t, size_t));
 void read_warn_details PARAMS ((char const *, off_t, size_t));
-void readdir_error PARAMS ((char const *));
-void readdir_warn PARAMS ((char const *));
 void readlink_error PARAMS ((char const *));
 void readlink_warn PARAMS ((char const *));
+void savedir_error PARAMS ((char const *));
+void savedir_warn PARAMS ((char const *));
 void seek_error PARAMS ((char const *));
 void seek_error_details PARAMS ((char const *, off_t));
 void seek_warn PARAMS ((char const *));
 void seek_warn_details PARAMS ((char const *, off_t));
 void stat_error PARAMS ((char const *));
 void stat_warn PARAMS ((char const *));
+void symlink_error PARAMS ((char const *, char const *));
 void truncate_error PARAMS ((char const *));
 void truncate_warn PARAMS ((char const *));
 void unlink_error PARAMS ((char const *));
@@ -542,6 +536,8 @@ char const *quote_n PARAMS ((int, char const *));
 
 /* Module names.c.  */
 
+extern struct name *gnu_list_name;
+
 void gid_to_gname PARAMS ((gid_t, char gname[GNAME_FIELD_SIZE]));
 int gname_to_gid PARAMS ((char gname[GNAME_FIELD_SIZE], gid_t *));
 void uid_to_uname PARAMS ((uid_t, char uname[UNAME_FIELD_SIZE]));
@@ -554,7 +550,7 @@ void name_term PARAMS ((void));
 char *name_next PARAMS ((int));
 void name_close PARAMS ((void));
 void name_gather PARAMS ((void));
-void addname PARAMS ((char const *, int));
+struct name *addname PARAMS ((char const *, int));
 int name_match PARAMS ((const char *));
 void names_notfound PARAMS ((void));
 void collect_and_sort_names PARAMS ((void));
