@@ -1,5 +1,5 @@
 /* Diff files from a tar archive.
-   Copyright 1988, 92, 93, 94, 96, 97, 99, 2000 Free Software Foundation, Inc.
+   Copyright 1988,92,93,94,96,97,99,2000, 2001 Free Software Foundation, Inc.
    Written by John Gilmore, on 1987-04-30.
 
    This program is free software; you can redistribute it and/or modify it
@@ -49,10 +49,7 @@ static int diff_handle;
 /* Area for reading file contents into.  */
 static char *diff_buffer;
 
-/*--------------------------------.
-| Initialize for a diff operation |
-`--------------------------------*/
-
+/* Initialize for a diff operation.  */
 void
 diff_init (void)
 {
@@ -61,11 +58,8 @@ diff_init (void)
     xalloc_die ();
 }
 
-/*------------------------------------------------------------------------.
-| Sigh about something that differs by writing a MESSAGE to stdlis, given |
-| MESSAGE is nonzero.  Also set the exit status if not already.           |
-`------------------------------------------------------------------------*/
-
+/* Sigh about something that differs by writing a MESSAGE to stdlis,
+   given MESSAGE is nonzero.  Also set the exit status if not already.  */
 static void
 report_difference (const char *message)
 {
@@ -76,23 +70,15 @@ report_difference (const char *message)
     exit_status = TAREXIT_DIFFERS;
 }
 
-/*-----------------------------------------------------------------------.
-| Takes a buffer returned by read_and_process and does nothing with it.	 |
-`-----------------------------------------------------------------------*/
-
-/* Yes, I know.  SIZE and DATA are unused in this function.  Some compilers
-   may even report it.  That's OK, just relax!  */
-
+/* Take a buffer returned by read_and_process and do nothing with it.  */
 static int
 process_noop (size_t size, char *data)
 {
+  /* Yes, I know.  SIZE and DATA are unused in this function.  Some
+     compilers may even report it.  That's OK, just relax!  */
   return 1;
 }
 
-/*---.
-| ?  |
-`---*/
-
 static int
 process_rawdata (size_t bytes, char *buffer)
 {
@@ -124,10 +110,6 @@ process_rawdata (size_t bytes, char *buffer)
   return 1;
 }
 
-/*---.
-| ?  |
-`---*/
-
 /* Directory contents, only for GNUTYPE_DUMPDIR.  */
 
 static char *dumpdir_cursor;
@@ -145,14 +127,11 @@ process_dumpdir (size_t bytes, char *buffer)
   return 1;
 }
 
-/*------------------------------------------------------------------------.
-| Some other routine wants SIZE bytes in the archive.  For each chunk of  |
-| the archive, call PROCESSOR with the size of the chunk, and the address |
-| of the chunk it can work with.  The PROCESSOR should return nonzero for |
-| success.  It it return error once, continue skipping without calling    |
-| PROCESSOR anymore.                                                      |
-`------------------------------------------------------------------------*/
-
+/* Some other routine wants SIZE bytes in the archive.  For each chunk
+   of the archive, call PROCESSOR with the size of the chunk, and the
+   address of the chunk it can work with.  The PROCESSOR should return
+   nonzero for success.  It it return error once, continue skipping
+   without calling PROCESSOR anymore.  */
 static void
 read_and_process (size_t size, int (*processor) (size_t, char *))
 {
@@ -183,17 +162,12 @@ read_and_process (size_t size, int (*processor) (size_t, char *))
     }
 }
 
-/*---.
-| ?  |
-`---*/
-
 /* JK This routine should be used more often than it is ... look into
    that.  Anyhow, what it does is translate the sparse information on the
    header, and in any subsequent extended headers, into an array of
    structures with true numbers, as opposed to character strings.  It
    simply makes our life much easier, doing so many comparisons and such.
    */
-
 static void
 fill_in_sparse_array (void)
 {
@@ -271,10 +245,6 @@ fill_in_sparse_array (void)
     }
 }
 
-/*---.
-| ?  |
-`---*/
-
 /* JK Diff'ing a sparse file with its counterpart on the tar file is a
    bit of a different story than a normal file.  First, we must know what
    areas of the file to skip through, i.e., we need to construct a
@@ -420,12 +390,9 @@ diff_sparse_files (off_t size_of_file)
     report_difference (_("Contents differ"));
 }
 
-/*---------------------------------------------------------------------.
-| Call either stat or lstat over STAT_DATA, depending on --dereference |
-| (-h), for a file which should exist.  Diagnose any problem.  Return  |
-| nonzero for success, zero otherwise.				       |
-`---------------------------------------------------------------------*/
-
+/* Call either stat or lstat over STAT_DATA, depending on
+   --dereference (-h), for a file which should exist.  Diagnose any
+   problem.  Return nonzero for success, zero otherwise.  */
 static int
 get_stat_data (char const *file_name, struct stat *stat_data)
 {
@@ -444,10 +411,7 @@ get_stat_data (char const *file_name, struct stat *stat_data)
   return 1;
 }
 
-/*----------------------------------.
-| Diff a file against the archive.  |
-`----------------------------------*/
-
+/* Diff a file against the archive.  */
 void
 diff_archive (void)
 {
@@ -459,7 +423,7 @@ diff_archive (void)
   set_next_block_after (current_header);
   decode_header (current_header, &current_stat, &current_format, 1);
 
-  /* Print the block from `current_header' and `current_stat'.  */
+  /* Print the block from current_header and current_stat.  */
 
   if (verbose_option)
     {
@@ -484,21 +448,19 @@ diff_archive (void)
       /* Appears to be a file.  See if it's really a directory.  */
 
       name_length = strlen (current_file_name) - 1;
-      if (current_file_name[name_length] == '/')
+      if (ISSLASH (current_file_name[name_length]))
 	goto really_dir;
 
       if (!get_stat_data (current_file_name, &stat_data))
 	{
-	  if (current_header->oldgnu_header.isextended)
-	    skip_extended_headers ();
-	  skip_file (current_stat.st_size);
+	  skip_member ();
 	  goto quit;
 	}
 
       if (!S_ISREG (stat_data.st_mode))
 	{
 	  report_difference (_("File type differs"));
-	  skip_file (current_stat.st_size);
+	  skip_member ();
 	  goto quit;
 	}
 
@@ -521,7 +483,7 @@ diff_archive (void)
 	  stat_data.st_size != current_stat.st_size)
 	{
 	  report_difference (_("Size differs"));
-	  skip_file (current_stat.st_size);
+	  skip_member ();
 	  goto quit;
 	}
 
@@ -530,9 +492,7 @@ diff_archive (void)
       if (diff_handle < 0)
 	{
 	  open_error (current_file_name);
-	  if (current_header->oldgnu_header.isextended)
-	    skip_extended_headers ();
-	  skip_file (current_stat.st_size);
+	  skip_member ();
 	  report_difference (0);
 	  goto quit;
 	}
@@ -599,9 +559,10 @@ diff_archive (void)
 #ifdef HAVE_READLINK
     case SYMTYPE:
       {
-	char linkbuf[NAME_FIELD_SIZE + 3]; /* FIXME: may be too short.  */
+	size_t len = strlen (current_link_name);
+	char *linkbuf = alloca (len + 1);
 
-	status = readlink (current_file_name, linkbuf, (sizeof linkbuf) - 1);
+	status = readlink (current_file_name, linkbuf, len + 1);
 
 	if (status < 0)
 	  {
@@ -610,11 +571,9 @@ diff_archive (void)
 	    else
 	      readlink_error (current_file_name);
 	    report_difference (0);
-	    break;
 	  }
-
-	linkbuf[status] = '\0';	/* null-terminate it */
-	if (strncmp (current_link_name, linkbuf, status) != 0)
+	else if (status != len
+		 || strncmp (current_link_name, linkbuf, len) != 0)
 	  report_difference (_("Symlink differs"));
 
 	break;
@@ -659,8 +618,7 @@ diff_archive (void)
 
     case GNUTYPE_DUMPDIR:
       {
-	char *dumpdir_buffer = get_directory_contents (current_file_name,
-						       (dev_t) 0);
+	char *dumpdir_buffer = get_directory_contents (current_file_name, 0);
 
 	if (multi_volume_option)
 	  {
@@ -689,7 +647,7 @@ diff_archive (void)
       name_length = strlen (current_file_name) - 1;
 
     really_dir:
-      while (name_length && current_file_name[name_length] == '/')
+      while (name_length && ISSLASH (current_file_name[name_length]))
 	current_file_name[name_length--] = '\0';	/* zap / */
 
       if (!get_stat_data (current_file_name, &stat_data))
@@ -717,7 +675,7 @@ diff_archive (void)
 	off_t offset;
 
 	name_length = strlen (current_file_name) - 1;
-	if (current_file_name[name_length] == '/')
+	if (ISSLASH (current_file_name[name_length]))
 	  goto really_dir;
 
 	if (!get_stat_data (current_file_name, &stat_data))
@@ -726,7 +684,7 @@ diff_archive (void)
 	if (!S_ISREG (stat_data.st_mode))
 	  {
 	    report_difference (_("File type differs"));
-	    skip_file (current_stat.st_size);
+	    skip_member ();
 	    break;
 	  }
 
@@ -734,7 +692,7 @@ diff_archive (void)
 	if (stat_data.st_size != current_stat.st_size + offset)
 	  {
 	    report_difference (_("Size differs"));
-	    skip_file (current_stat.st_size);
+	    skip_member ();
 	    break;
 	  }
 
@@ -744,7 +702,7 @@ diff_archive (void)
 	  {
 	    open_error (current_file_name);
 	    report_difference (0);
-	    skip_file (current_stat.st_size);
+	    skip_member ();
 	    break;
 	  }
 
@@ -776,10 +734,6 @@ diff_archive (void)
     }
 }
 
-/*---.
-| ?  |
-`---*/
-
 void
 verify_volume (void)
 {
