@@ -1,5 +1,5 @@
 /* Miscellaneous functions, not really specific to GNU tar.
-   Copyright 1988, 92, 94, 95, 96, 97, 99, 2000 Free Software Foundation, Inc.
+   Copyright 1988,92,94,95,96,97,99,2000, 2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -26,11 +26,8 @@ static void call_arg_fatal PARAMS ((char const *, char const *))
 
 /* Handling strings.  */
 
-/*-------------------------------------------------------------------------.
-| Assign STRING to a copy of VALUE if not zero, or to zero.  If STRING was |
-| nonzero, it is freed first.						   |
-`-------------------------------------------------------------------------*/
-
+/* Assign STRING to a copy of VALUE if not zero, or to zero.  If
+   STRING was nonzero, it is freed first.  */
 void
 assign_string (char **string, const char *value)
 {
@@ -39,20 +36,18 @@ assign_string (char **string, const char *value)
   *string = value ? xstrdup (value) : 0;
 }
 
-/*------------------------------------------------------------------------.
-| Allocate a copy of the string quoted as in C, and returns that.  If the |
-| string does not have to be quoted, it returns a null pointer.  The	  |
-| allocated copy should normally be freed with free() after the caller is |
-| done with it.								  |
-| 									  |
-| This is used in one context only: generating the directory file in	  |
-| incremental dumps.  The quoted string is not intended for human	  |
-| consumption; it is intended only for unquote_string.  The quoting	  |
-| is locale-independent, so that users needn't worry about locale when	  |
-| reading directory files.  This means that we can't use quotearg, as	  |
-| quotearg is locale-dependent and is meant for human consumption.	  |
-`------------------------------------------------------------------------*/
+/* Allocate a copy of the string quoted as in C, and returns that.  If
+   the string does not have to be quoted, it returns a null pointer.
+   The allocated copy should normally be freed with free() after the
+   caller is done with it.
 
+   This is used in one context only: generating the directory file in
+   incremental dumps.  The quoted string is not intended for human
+   consumption; it is intended only for unquote_string.  The quoting
+   is locale-independent, so that users needn't worry about locale
+   when reading directory files.  This means that we can't use
+   quotearg, as quotearg is locale-dependent and is meant for human
+   consumption.  */
 char *
 quote_copy_string (const char *string)
 {
@@ -95,18 +90,16 @@ quote_copy_string (const char *string)
   return 0;
 }
 
-/*-------------------------------------------------------------------------.
-| Takes a quoted C string (like those produced by quote_copy_string) and   |
-| turns it back into the un-quoted original.  This is done in place.	   |
-| Returns 0 only if the string was not properly quoted, but completes the  |
-| unquoting anyway.							   |
-| 									   |
-| This is used for reading the saved directory file in incremental dumps.  |
-| It is used for decoding old `N' records (demangling names).  But also,   |
-| it is used for decoding file arguments, would they come from the shell   |
-| or a -T file, and for decoding the --exclude argument.		   |
-`-------------------------------------------------------------------------*/
-
+/* Takes a quoted C string (like those produced by quote_copy_string)
+   and turns it back into the un-quoted original.  This is done in
+   place.  Returns 0 only if the string was not properly quoted, but
+   completes the unquoting anyway.
+ 									   
+   This is used for reading the saved directory file in incremental
+   dumps.  It is used for decoding old `N' records (demangling names).
+   But also, it is used for decoding file arguments, would they come
+   from the shell or a -T file, and for decoding the --exclude
+   argument.  */
 int
 unquote_string (char *string)
 {
@@ -202,15 +195,24 @@ unquote_string (char *string)
   return result;
 }
 
-/* Return nonzero if P contains ".." as a path name component.  */
+/* Return nonzero if NAME contains ".." as a path name component.  */
 int
-contains_dot_dot (char const *p)
+contains_dot_dot (char const *name)
 {
-  for (;; p++)
-    if (p[0] == '.' && p[1] == '.' && (p[2] == '/' || !p[2]))
-      return 1;
-    else if (! (p = strchr (p, '/')))
-      return 0;
+  char const *p = name + FILESYSTEM_PREFIX_LEN (name);
+
+  for (;;)
+    {
+      if (p[0] == '.' && p[1] == '.' && (ISSLASH (p[2]) || !p[2]))
+	return 1;
+
+      do
+	{
+	  if (! *p++)
+	    return 0;
+	}
+      while (! ISSLASH (*p));
+    }
 }
 
 /* File handling.  */
@@ -219,28 +221,20 @@ contains_dot_dot (char const *p)
 static char *before_backup_name;
 static char *after_backup_name;
 
-/*------------------------------------------------------------------------.
-| Returns nonzero if p is `.' or `..'.  This could be a macro for speed.  |
-`------------------------------------------------------------------------*/
-
-/* Early Solaris 2.4 readdir may return d->d_name as `' in NFS-mounted
-   directories.  The workaround here skips `' just like `.'.  Without it,
-   GNU tar would then treat `' much like `.' and loop endlessly.  */
-
-int
-is_dot_or_dotdot (const char *p)
-{
-  return (p[0] == '\0'
-	  || (p[0] == '.' && (p[1] == '\0'
-			      || (p[1] == '.' && p[2] == '\0'))));
-}
-
 /* Some implementations of rmdir let you remove the working directory.
    Report an error with errno set to zero for obvious cases of this;
    otherwise call rmdir.  */
 static int
 safer_rmdir (const char *path)
 {
+  while (path[0] == '.' && ISSLASH (path[1]))
+    {
+      path++;
+
+      do path++;
+      while (ISSLASH (*path));
+    }
+
   if (! path[0] || (path[0] == '.' && ! path[1]))
     {
       errno = 0;
@@ -250,61 +244,64 @@ safer_rmdir (const char *path)
   return rmdir (path);
 }
 
-/*-------------------------------------------------------------------------.
-| Delete PATH, whatever it might be.  If RECURSE, first recursively delete |
-| the contents of PATH when it is a directory.  Return zero on any error,  |
-| with errno set.  Attempting to remove the working directory is an error, |
-| with errno set to zero.						   |
-`-------------------------------------------------------------------------*/
-
+/* Remove PATH.  If PATH is a directory, then if RECURSE is set remove
+   it recursively; otherwise, remove it only if it is empty.  Return 0
+   on error, with errno set; if PATH is obviously the working
+   directory return zero with errno set to zero.  */
 int
 remove_any_file (const char *path, int recurse)
 {
+  /* Try unlink first if we are not root, as this saves us a system
+     call in the common case where we're removing a non-directory.  */
+  if (! we_are_root)
+    {
+      if (unlink (path) == 0)
+	return 1;
+      if (errno != EPERM)
+	return 0;
+    }
+
   if (safer_rmdir (path) == 0)
     return 1;
 
   switch (errno)
     {
     case ENOTDIR:
-      return unlink (path) == 0;
+      return we_are_root && unlink (path) == 0;
 
     case 0:
     case EEXIST:
-#ifdef ENOTEMPTY
+#if defined ENOTEMPTY && ENOTEMPTY != EEXIST
     case ENOTEMPTY:
 #endif
       if (recurse)
 	{
-	  DIR *dirp = opendir (path);
-	  struct dirent *dp;
+	  char *directory = savedir (path);
+	  char const *entry;
+	  size_t entrylen;
 
-	  if (! dirp)
+	  if (! directory)
 	    return 0;
 
-	  while (errno = 0, (dp = readdir (dirp)))
-	    if (! is_dot_or_dotdot (dp->d_name))
-	      {
-		char *path_buffer = new_name (path, dp->d_name);
-		int r = remove_any_file (path_buffer, 1);
-		int e = errno;
-		free (path_buffer);
-
-		if (! r)
-		  {
-		    errno = e;
-		    break;
-		  }
-	      }
-
-	  if (errno)
+	  for (entry = directory;
+	       (entrylen = strlen (entry)) != 0;
+	       entry += entrylen + 1)
 	    {
+	      char *path_buffer = new_name (path, entry);
+	      int r = remove_any_file (path_buffer, 1);
 	      int e = errno;
-	      closedir (dirp);
-	      errno = e;
-	      return 0;
+	      free (path_buffer);
+
+	      if (! r)
+		{
+		  free (directory);
+		  errno = e;
+		  return 0;
+		}
 	    }
 
-	  return closedir (dirp) == 0 && safer_rmdir (path) == 0;
+	  free (directory);
+	  return safer_rmdir (path) == 0;
 	}
       break;
     }
@@ -312,15 +309,11 @@ remove_any_file (const char *path, int recurse)
   return 0;
 }
 
-/*-------------------------------------------------------------------------.
-| Check if PATH already exists and make a backup of it right now.  Return  |
-| success (nonzero) only if the backup in either unneeded, or successful.  |
-| 									   |
-| For now, directories are considered to never need backup.  If ARCHIVE is |
-| nonzero, this is the archive and so, we do not have to backup block or   |
-| character devices, nor remote entities.				   |
-`-------------------------------------------------------------------------*/
-
+/* Check if PATH already exists and make a backup of it right now.
+   Return success (nonzero) only if the backup in either unneeded, or
+   successful.  For now, directories are considered to never need
+   backup.  If ARCHIVE is nonzero, this is the archive and so, we do
+   not have to backup block or character devices, nor remote entities.  */
 int
 maybe_backup_file (const char *path, int archive)
 {
@@ -378,11 +371,8 @@ maybe_backup_file (const char *path, int archive)
     }
 }
 
-/*-----------------------------------------------------------------------.
-| Try to restore the recently backed up file to its original name.  This |
-| is usually only needed after a failed extraction.			 |
-`-----------------------------------------------------------------------*/
-
+/* Try to restore the recently backed up file to its original name.
+   This is usually only needed after a failed extraction.  */
 void
 undo_last_backup (void)
 {
@@ -589,18 +579,6 @@ close_warn (char const *name)
   call_arg_warn ("close", name);
 }
 
-void
-closedir_error (char const *name)
-{
-  call_arg_error ("closedir", name);
-}
-
-void
-closedir_warn (char const *name)
-{
-  call_arg_warn ("closedir", name);
-}
-
 void
 exec_fatal (char const *name)
 {
@@ -643,18 +621,6 @@ open_warn (char const *name)
   call_arg_warn ("open", name);
 }
 
-void
-opendir_error (char const *name)
-{
-  call_arg_error ("opendir", name);
-}
-
-void
-opendir_warn (char const *name)
-{
-  call_arg_warn ("opendir", name);
-}
-
 void
 read_error (char const *name)
 {
@@ -668,7 +634,8 @@ read_error_details (char const *name, off_t offset, size_t size)
   int e = errno;
   ERROR ((0, e,
 	  _("%s: Read error at byte %s, reading %lu bytes"),
-	  quotearg_colon (name), STRINGIFY_BIGINT (offset, buf), size));
+	  quotearg_colon (name), STRINGIFY_BIGINT (offset, buf),
+	  (unsigned long) size));
 }
 
 void
@@ -678,7 +645,8 @@ read_warn_details (char const *name, off_t offset, size_t size)
   int e = errno;
   WARN ((0, e,
 	 _("%s: Warning: Read error at byte %s, reading %lu bytes"),
-	 quotearg_colon (name), STRINGIFY_BIGINT (offset, buf), size));
+	 quotearg_colon (name), STRINGIFY_BIGINT (offset, buf),
+	 (unsigned long) size));
 }
 
 void
@@ -694,19 +662,8 @@ read_fatal_details (char const *name, off_t offset, size_t size)
   int e = errno;
   FATAL_ERROR ((0, e,
 		_("%s: Read error at byte %s, reading %lu bytes"),
-		quotearg_colon (name), STRINGIFY_BIGINT (offset, buf), size));
-}
-
-void
-readdir_error (char const *name)
-{
-  call_arg_error ("readdir", name);
-}
-
-void
-readdir_warn (char const *name)
-{
-  call_arg_warn ("readdir", name);
+		quotearg_colon (name), STRINGIFY_BIGINT (offset, buf),
+		(unsigned long) size));
 }
 
 void
@@ -721,6 +678,18 @@ readlink_warn (char const *name)
   call_arg_warn ("readlink", name);
 }
 
+void
+savedir_error (char const *name)
+{
+  call_arg_error ("savedir", name);
+}
+
+void
+savedir_warn (char const *name)
+{
+  call_arg_warn ("savedir", name);
+}
+
 void
 seek_error (char const *name)
 {
@@ -753,6 +722,14 @@ seek_warn_details (char const *name, off_t offset)
 	 STRINGIFY_BIGINT (offset, buf)));
 }
 
+void
+symlink_error (char const *contents, char const *name)
+{
+  int e = errno;
+  ERROR ((0, e, _("%s: Cannot create symlink to %s"),
+	  quotearg_colon (name), quote_n (1, contents)));
+}
+
 void
 stat_error (char const *name)
 {
@@ -843,7 +820,7 @@ xpipe (int fd[2])
     call_arg_fatal ("pipe", _("interprocess channel"));
 }
 
-/* Return an unambiguous printable representated, allocated in slot N,
+/* Return an unambiguous printable representation, allocated in slot N,
    for NAME, suitable for diagnostics.  */
 char const *
 quote_n (int n, char const *name)
