@@ -1,5 +1,5 @@
 /* System dependent definitions for GNU tar.
-   Copyright 1994, 1995, 1996, 1997, 1998, 1999 Free Software Foundation, Inc.
+   Copyright 1994,95,96,97,98,99,2000, 2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -48,18 +48,18 @@ char *alloca ();
 #include <sys/types.h>
 #include <ctype.h>
 
-/* CTYPE_DOMAIN (C) is nonzero if the unsigned char C can safely be given
+/* IN_CTYPE_DOMAIN (C) is nonzero if the unsigned char C can safely be given
    as an argument to <ctype.h> macros like `isspace'.  */
 #if STDC_HEADERS
-# define CTYPE_DOMAIN(c) 1
+# define IN_CTYPE_DOMAIN(c) 1
 #else
-# define CTYPE_DOMAIN(c) ((unsigned) (c) <= 0177)
+# define IN_CTYPE_DOMAIN(c) ((unsigned) (c) <= 0177)
 #endif
 
 #define ISDIGIT(c) ((unsigned) (c) - '0' <= 9)
 #define ISODIGIT(c) ((unsigned) (c) - '0' <= 7)
-#define ISPRINT(c) (CTYPE_DOMAIN (c) && isprint (c))
-#define ISSPACE(c) (CTYPE_DOMAIN (c) && isspace (c))
+#define ISPRINT(c) (IN_CTYPE_DOMAIN (c) && isprint (c))
+#define ISSPACE(c) (IN_CTYPE_DOMAIN (c) && isspace (c))
 
 #ifndef FILESYSTEM_PREFIX_LEN
 # define FILESYSTEM_PREFIX_LEN(filename) 0
@@ -134,10 +134,6 @@ extern int errno;
 				/* MS-DOG forever, with my love! */
 #ifndef	O_BINARY
 # define O_BINARY 0
-#endif
-				/* Emulate System V 3-argument open call */
-#if EMUL_OPEN3
-# define open open3
 #endif
 
 /* Declare file status routines and bits.  */
@@ -350,34 +346,6 @@ extern int errno;
 
 #undef GOT_MAJOR
 
-/* Declare directory reading routines and structures.  */
-
-#if __TURBOC__
-# include "msd_dir.h"
-# define NAMLEN(dirent) ((dirent)->d_namlen)
-#else
-# if HAVE_DIRENT_H
-#  include <dirent.h>
-#  define NAMLEN(dirent) (strlen((dirent)->d_name))
-# else
-#  define dirent direct
-#  define NAMLEN(dirent) ((dirent)->d_namlen)
-#  if HAVE_SYS_NDIR_H
-#   include <sys/ndir.h>
-#  endif
-#  if HAVE_SYS_DIR_H
-#   include <sys/dir.h>
-#  endif
-#  if HAVE_NDIR_H
-#   include <ndir.h>
-#  endif
-# endif
-#endif
-
-#if CLOSEDIR_VOID
-# define closedir(dirp) ((closedir) (dirp), 0)
-#endif
-
 /* Declare wait status.  */
 
 #if HAVE_SYS_WAIT_H
@@ -493,6 +461,12 @@ void *realloc ();
 char *getenv ();
 #endif
 
+#if HAVE_STDBOOL_H
+# include <stdbool.h>
+#else
+typedef enum {false = 0, true = 1} bool;
+#endif
+
 #include <stdio.h>
 
 #ifndef _POSIX_VERSION
@@ -503,8 +477,6 @@ off_t lseek ();
 # endif
 #endif
 
-#include <pathmax.h>
-
 #if WITH_DMALLOC
 # undef HAVE_VALLOC
 # define DMALLOC_FUNC_CHECK
@@ -531,17 +503,6 @@ off_t lseek ();
 # define LONG_MAX TYPE_MAXIMUM (long)
 #endif
 
-#if HAVE_SIGNED_CHAR
-# define signed_char(x) ((signed char) (x))
-#else
-# define signed_char(x) \
-    ('\200' < 0 \
-     ? (char) (x) \
-     : ((char) (x) \
-        << CHAR_BIT * (sizeof (int) - 1) \
-	>> CHAR_BIT * (sizeof (int) - 1)))
-#endif
-
 #if HAVE_INTTYPES_H
 # include <inttypes.h>
 #endif
@@ -594,10 +555,16 @@ off_t lseek ();
 #endif
 #define N_(Text) Text
 
+#include <time.h>
+#ifndef time
+time_t time ();
+#endif
+
 /* Library modules.  */
 
-#include "error.h"
-#include "xalloc.h"
+#include <error.h>
+#include <savedir.h>
+#include <xalloc.h>
 
 #if !HAVE_STRSTR
 char *strstr PARAMS ((const char *, const char *));
